/*
Copyright (c) 2017-2019,
Battelle Memorial Institute; Lawrence Livermore National Security, LLC; Alliance for Sustainable
Energy, LLC.  See the top-level NOTICE for additional details. All rights reserved.
SPDX-License-Identifier: BSD-3-Clause
*/

/** @file
Header file for inclusion of the entire CPP shared library
*/
#ifndef SRC_HELICS_HELICS98_HPP_
#define SRC_HELICS_HELICS98_HPP_

#include "cpp98/Broker.hpp"
#include "cpp98/Core.hpp"
#include "cpp98/helics.hpp"

#endif  // SRC_HELICS_HELICS98_HPP_
